@@ -5,6 +5,7 @@ import (
 	"context"
 	"database/sql"
 	"encoding/json"
+	"errors"
 	"fmt"
 	"io"
 	"net/http"
@@ -69,18 +70,71 @@ var apiInternal = []APIEndpoint{
 	internalWarningCreateCmd,
 }
 
-var internalShutdownCmd = APIEndpoint{
-	Path: "shutdown",
-
-	Put: APIEndpointAction{Handler: internalShutdown, AccessHandler: allowPermission(auth.ObjectTypeServer, auth.EntitlementCanEdit)},
-}
-
+// Daemon management internal commands.
 var internalReadyCmd = APIEndpoint{
 	Path: "ready",
 
 	Get: APIEndpointAction{Handler: internalWaitReady, AccessHandler: allowPermission(auth.ObjectTypeServer, auth.EntitlementCanEdit)},
 }
 
+var internalShutdownCmd = APIEndpoint{
+	Path: "shutdown",
+
+	Put: APIEndpointAction{Handler: internalShutdown, AccessHandler: allowPermission(auth.ObjectTypeServer, auth.EntitlementCanEdit)},
+}
+
+// Internal managemnt traffic.
+var internalImageOptimizeCmd = APIEndpoint{
+	Path: "image-optimize",
+
+	Post: APIEndpointAction{Handler: internalOptimizeImage, AccessHandler: allowPermission(auth.ObjectTypeServer, auth.EntitlementCanEdit)},
+}
+
+var internalRebalanceLoadCmd = APIEndpoint{
+	Path: "rebalance",
+
+	Get: APIEndpointAction{Handler: internalRebalanceLoad, AccessHandler: allowPermission(auth.ObjectTypeServer, auth.EntitlementCanEdit)},
+}
+
+var internalSQLCmd = APIEndpoint{
+	Path: "sql",
+
+	Get:  APIEndpointAction{Handler: internalSQLGet, AccessHandler: allowPermission(auth.ObjectTypeServer, auth.EntitlementCanEdit)},
+	Post: APIEndpointAction{Handler: internalSQLPost, AccessHandler: allowPermission(auth.ObjectTypeServer, auth.EntitlementCanEdit)},
+}
+
+// Internal cluster traffic.
+var internalClusterAcceptCmd = APIEndpoint{
+	Path: "cluster/accept",
+
+	Post: APIEndpointAction{Handler: internalClusterPostAccept, AccessHandler: allowPermission(auth.ObjectTypeServer, auth.EntitlementCanEdit)},
+}
+
+var internalClusterAssignCmd = APIEndpoint{
+	Path: "cluster/assign",
+
+	Post: APIEndpointAction{Handler: internalClusterPostAssign, AccessHandler: allowPermission(auth.ObjectTypeServer, auth.EntitlementCanEdit)},
+}
+
+var internalClusterHandoverCmd = APIEndpoint{
+	Path: "cluster/handover",
+
+	Post: APIEndpointAction{Handler: internalClusterPostHandover, AccessHandler: allowPermission(auth.ObjectTypeServer, auth.EntitlementCanEdit)},
+}
+
+var internalClusterRaftNodeCmd = APIEndpoint{
+	Path: "cluster/raft-node/{address}",
+
+	Delete: APIEndpointAction{Handler: internalClusterRaftNodeDelete, AccessHandler: allowPermission(auth.ObjectTypeServer, auth.EntitlementCanEdit)},
+}
+
+var internalClusterRebalanceCmd = APIEndpoint{
+	Path: "cluster/rebalance",
+
+	Post: APIEndpointAction{Handler: internalClusterPostRebalance, AccessHandler: allowPermission(auth.ObjectTypeServer, auth.EntitlementCanEdit)},
+}
+
+// Container hooks.
 var internalContainerOnStartCmd = APIEndpoint{
 	Path: "containers/{instanceRef}/onstart",
 
@@ -99,59 +153,42 @@ var internalContainerOnStopCmd = APIEndpoint{
 	Get: APIEndpointAction{Handler: internalContainerOnStop, AccessHandler: allowPermission(auth.ObjectTypeServer, auth.EntitlementCanEdit)},
 }
 
+// Virtual machine hooks.
 var internalVirtualMachineOnResizeCmd = APIEndpoint{
 	Path: "virtual-machines/{instanceRef}/onresize",
 
 	Get: APIEndpointAction{Handler: internalVirtualMachineOnResize, AccessHandler: allowPermission(auth.ObjectTypeServer, auth.EntitlementCanEdit)},
 }
 
-var internalSQLCmd = APIEndpoint{
-	Path: "sql",
-
-	Get:  APIEndpointAction{Handler: internalSQLGet, AccessHandler: allowPermission(auth.ObjectTypeServer, auth.EntitlementCanEdit)},
-	Post: APIEndpointAction{Handler: internalSQLPost, AccessHandler: allowPermission(auth.ObjectTypeServer, auth.EntitlementCanEdit)},
-}
-
-var internalGarbageCollectorCmd = APIEndpoint{
-	Path: "gc",
-
-	Get: APIEndpointAction{Handler: internalGC, AccessHandler: allowPermission(auth.ObjectTypeServer, auth.EntitlementCanEdit)},
-}
-
-var internalRAFTSnapshotCmd = APIEndpoint{
-	Path: "raft-snapshot",
-
-	Get: APIEndpointAction{Handler: internalRAFTSnapshot, AccessHandler: allowPermission(auth.ObjectTypeServer, auth.EntitlementCanEdit)},
-}
-
-var internalImageRefreshCmd = APIEndpoint{
-	Path: "testing/image-refresh",
-
-	Get: APIEndpointAction{Handler: internalRefreshImage, AccessHandler: allowPermission(auth.ObjectTypeServer, auth.EntitlementCanEdit)},
-}
-
-var internalImageOptimizeCmd = APIEndpoint{
-	Path: "image-optimize",
-
-	Post: APIEndpointAction{Handler: internalOptimizeImage, AccessHandler: allowPermission(auth.ObjectTypeServer, auth.EntitlementCanEdit)},
-}
-
-var internalWarningCreateCmd = APIEndpoint{
-	Path: "testing/warnings",
-
-	Post: APIEndpointAction{Handler: internalCreateWarning, AccessHandler: allowPermission(auth.ObjectTypeServer, auth.EntitlementCanEdit)},
-}
-
+// Debugging.
 var internalBGPStateCmd = APIEndpoint{
-	Path: "testing/bgp",
+	Path: "debug/bgp",
 
 	Get: APIEndpointAction{Handler: internalBGPState, AccessHandler: allowPermission(auth.ObjectTypeServer, auth.EntitlementCanEdit)},
 }
 
-var internalRebalanceLoadCmd = APIEndpoint{
-	Path: "rebalance",
+var internalGarbageCollectorCmd = APIEndpoint{
+	Path: "debug/gc",
 
-	Get: APIEndpointAction{Handler: internalRebalanceLoad, AccessHandler: allowPermission(auth.ObjectTypeServer, auth.EntitlementCanEdit)},
+	Get: APIEndpointAction{Handler: internalGC, AccessHandler: allowPermission(auth.ObjectTypeServer, auth.EntitlementCanEdit)},
+}
+
+var internalImageRefreshCmd = APIEndpoint{
+	Path: "debug/image-refresh",
+
+	Get: APIEndpointAction{Handler: internalRefreshImage, AccessHandler: allowPermission(auth.ObjectTypeServer, auth.EntitlementCanEdit)},
+}
+
+var internalRAFTSnapshotCmd = APIEndpoint{
+	Path: "debug/raft-snapshot",
+
+	Get: APIEndpointAction{Handler: internalRAFTSnapshot, AccessHandler: allowPermission(auth.ObjectTypeServer, auth.EntitlementCanEdit)},
+}
+
+var internalWarningCreateCmd = APIEndpoint{
+	Path: "debug/warnings",
+
+	Post: APIEndpointAction{Handler: internalCreateWarning, AccessHandler: allowPermission(auth.ObjectTypeServer, auth.EntitlementCanEdit)},
 }
 
 type internalImageOptimizePost struct {
@@ -196,7 +233,7 @@ func internalCreateWarning(d *Daemon, r *http.Request) response.Response {
 	// Check if the entity exists, and fail if it doesn't.
 	_, ok := cluster.EntityNames[req.EntityTypeCode]
 	if req.EntityTypeCode != -1 && !ok {
-		return response.SmartError(fmt.Errorf("Invalid entity type"))
+		return response.SmartError(errors.New("Invalid entity type"))
 	}
 
 	err = d.State().DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
@@ -228,7 +265,7 @@ func internalOptimizeImage(d *Daemon, r *http.Request) response.Response {
 	return response.EmptySyncResponse
 }
 
-func internalRefreshImage(d *Daemon, r *http.Request) response.Response {
+func internalRefreshImage(d *Daemon, _ *http.Request) response.Response {
 	s := d.State()
 
 	err := autoUpdateImages(s.ShutdownCtx, s)
@@ -239,15 +276,15 @@ func internalRefreshImage(d *Daemon, r *http.Request) response.Response {
 	return response.EmptySyncResponse
 }
 
-func internalWaitReady(d *Daemon, r *http.Request) response.Response {
+func internalWaitReady(d *Daemon, _ *http.Request) response.Response {
 	// Check that we're not shutting down.
 	isClosing := d.State().ShutdownCtx.Err() != nil
 	if isClosing {
-		return response.Unavailable(fmt.Errorf("Daemon is shutting down"))
+		return response.Unavailable(errors.New("Daemon is shutting down"))
 	}
 
 	if d.waitReady.Err() == nil {
-		return response.Unavailable(fmt.Errorf("Daemon not ready yet"))
+		return response.Unavailable(errors.New("Daemon not ready yet"))
 	}
 
 	return response.EmptySyncResponse
@@ -284,7 +321,7 @@ func internalShutdown(d *Daemon, r *http.Request) response.Response {
 		if ok {
 			f.Flush()
 		} else {
-			return fmt.Errorf("http.ResponseWriter is not type http.Flusher")
+			return errors.New("http.ResponseWriter is not type http.Flusher")
 		}
 
 		// Send result of d.Stop() to cmdDaemon so that process stops with correct exit code from Stop().
@@ -333,7 +370,7 @@ func internalContainerHookLoadFromReference(s *state.State, r *http.Request) (in
 	}
 
 	if inst.Type() != instancetype.Container {
-		return nil, fmt.Errorf("Instance is not container type")
+		return nil, errors.New("Instance is not container type")
 	}
 
 	return inst, nil
@@ -358,6 +395,9 @@ func internalContainerOnStart(d *Daemon, r *http.Request) response.Response {
 }
 
 func internalContainerOnStopNS(d *Daemon, r *http.Request) response.Response {
+	// Wait until daemon is fully started.
+	<-d.waitReady.Done()
+
 	s := d.State()
 
 	inst, err := internalContainerHookLoadFromReference(s, r)
@@ -388,6 +428,9 @@ func internalContainerOnStopNS(d *Daemon, r *http.Request) response.Response {
 }
 
 func internalContainerOnStop(d *Daemon, r *http.Request) response.Response {
+	// Wait until daemon is fully started.
+	<-d.waitReady.Done()
+
 	s := d.State()
 
 	inst, err := internalContainerHookLoadFromReference(s, r)
@@ -415,6 +458,9 @@ func internalContainerOnStop(d *Daemon, r *http.Request) response.Response {
 }
 
 func internalVirtualMachineOnResize(d *Daemon, r *http.Request) response.Response {
+	// Wait until daemon is fully started.
+	<-d.waitReady.Done()
+
 	s := d.State()
 
 	// Get the instance ID.
@@ -426,7 +472,7 @@ func internalVirtualMachineOnResize(d *Daemon, r *http.Request) response.Respons
 	// Get the devices list.
 	devices := request.QueryParam(r, "devices")
 	if devices == "" {
-		return response.BadRequest(fmt.Errorf("Resize hook requires a list of devices"))
+		return response.BadRequest(errors.New("Resize hook requires a list of devices"))
 	}
 
 	// Load by ID.
@@ -471,15 +517,17 @@ func internalSQLGet(d *Daemon, r *http.Request) response.Response {
 	database := r.FormValue("database")
 
 	if !slices.Contains([]string{"local", "global"}, database) {
-		return response.BadRequest(fmt.Errorf("Invalid database"))
+		return response.BadRequest(errors.New("Invalid database"))
 	}
 
-	schemaFormValue := r.FormValue("schema")
-	schemaOnly, err := strconv.Atoi(schemaFormValue)
+	dumpFormValue := r.FormValue("dump")
+	dumpInt, err := strconv.Atoi(dumpFormValue)
 	if err != nil {
-		schemaOnly = 0
+		dumpInt = 0
 	}
 
+	dumpOption := query.DumpOptions(dumpInt)
+
 	var db *sql.DB
 	if database == "global" {
 		db = s.DB.Cluster.DB()
@@ -494,7 +542,7 @@ func internalSQLGet(d *Daemon, r *http.Request) response.Response {
 
 	defer func() { _ = tx.Rollback() }()
 
-	dump, err := query.Dump(r.Context(), tx, schemaOnly == 1)
+	dump, err := query.Dump(r.Context(), tx, dumpOption)
 	if err != nil {
 		return response.SmartError(fmt.Errorf("Failed dump database %s: %w", database, err))
 	}
@@ -514,11 +562,11 @@ func internalSQLPost(d *Daemon, r *http.Request) response.Response {
 	}
 
 	if !slices.Contains([]string{"local", "global"}, req.Database) {
-		return response.BadRequest(fmt.Errorf("Invalid database"))
+		return response.BadRequest(errors.New("Invalid database"))
 	}
 
 	if req.Query == "" {
-		return response.BadRequest(fmt.Errorf("No query provided"))
+		return response.BadRequest(errors.New("No query provided"))
 	}
 
 	var db *sql.DB
@@ -634,9 +682,9 @@ func internalSQLExec(tx *sql.Tx, query string, result *internalSQL.SQLResult) er
 
 // internalImportFromBackup creates instance, storage pool and volume DB records from an instance's backup file.
 // It expects the instance volume to be mounted so that the backup.yaml file is readable.
-func internalImportFromBackup(ctx context.Context, s *state.State, projectName string, instName string, allowNameOverride bool) error {
+func internalImportFromBackup(ctx context.Context, s *state.State, projectName string, instName string, allowNameOverride bool, deviceMap map[string]map[string]string, configMap map[string]string) error {
 	if instName == "" {
-		return fmt.Errorf("The name of the instance is required")
+		return errors.New("The name of the instance is required")
 	}
 
 	storagePoolsPath := internalUtil.VarPath("storage-pools")
@@ -707,7 +755,7 @@ func internalImportFromBackup(ctx context.Context, s *state.State, projectName s
 		return err
 	}
 
-	if allowNameOverride && instName != "" {
+	if allowNameOverride {
 		backupConf.Container.Name = instName
 	}
 
@@ -717,7 +765,7 @@ func internalImportFromBackup(ctx context.Context, s *state.State, projectName s
 
 	if backupConf.Pool == nil {
 		// We don't know what kind of storage type the pool is.
-		return fmt.Errorf("No storage pool struct in the backup file found. The storage pool needs to be recovered manually")
+		return errors.New("No storage pool struct in the backup file found. The storage pool needs to be recovered manually")
 	}
 
 	// Try to retrieve the storage pool the instance supposedly lives on.
@@ -784,7 +832,7 @@ func internalImportFromBackup(ctx context.Context, s *state.State, projectName s
 	}
 
 	if backupConf.Volume == nil {
-		return fmt.Errorf(`No storage volume struct in the backup file found. The storage volume needs to be recovered manually`)
+		return errors.New(`No storage volume struct in the backup file found. The storage volume needs to be recovered manually`)
 	}
 
 	var profiles []api.Profile
@@ -798,22 +846,49 @@ func internalImportFromBackup(ctx context.Context, s *state.State, projectName s
 		return fmt.Errorf("Failed loading profiles (%v) for instance: %w", strings.Join(backupConf.Container.Profiles, ", "), err)
 	}
 
-	// Add root device if needed.
+	// Initialize configuration if missing.
+	if backupConf.Container.Config == nil {
+		backupConf.Container.Config = map[string]string{}
+	}
+
 	if backupConf.Container.Devices == nil {
-		backupConf.Container.Devices = make(map[string]map[string]string, 0)
+		backupConf.Container.Devices = make(map[string]map[string]string)
 	}
 
 	if backupConf.Container.ExpandedDevices == nil {
-		backupConf.Container.ExpandedDevices = make(map[string]map[string]string, 0)
+		backupConf.Container.ExpandedDevices = make(map[string]map[string]string)
 	}
 
+	// Add root device if needed.
 	internalImportRootDevicePopulate(instancePoolName, backupConf.Container.Devices, backupConf.Container.ExpandedDevices, profiles)
 
-	revert := revert.New()
-	defer revert.Fail()
+	// Override device.
+	for k, m := range deviceMap {
+		for key, value := range m {
+			if backupConf.Container.Devices[k] == nil {
+				backupConf.Container.Devices[k] = map[string]string{}
+			}
+
+			if backupConf.Container.ExpandedDevices[k] == nil {
+				backupConf.Container.ExpandedDevices[k] = map[string]string{}
+			}
+
+			backupConf.Container.Devices[k][key] = value
+			backupConf.Container.ExpandedDevices[k][key] = value
+		}
+	}
+
+	// Override config.
+	for key, value := range configMap {
+		backupConf.Container.Config[key] = value
+		backupConf.Container.ExpandedConfig[key] = value
+	}
+
+	reverter := revert.New()
+	defer reverter.Fail()
 
 	if backupConf.Container == nil {
-		return fmt.Errorf("No instance config in backup config")
+		return errors.New("No instance config in backup config")
 	}
 
 	instDBArgs, err := backup.ConfigToInstanceDBArgs(s, backupConf, projectName, true)
@@ -826,7 +901,7 @@ func internalImportFromBackup(ctx context.Context, s *state.State, projectName s
 		return fmt.Errorf("Failed creating instance record: %w", err)
 	}
 
-	revert.Add(cleanup)
+	reverter.Add(cleanup)
 	defer instOp.Done(err)
 
 	instancePath := storagePools.InstancePath(instanceType, projectName, backupConf.Container.Name, false)
@@ -871,32 +946,14 @@ func internalImportFromBackup(ctx context.Context, s *state.State, projectName s
 			return err
 		}
 
-		// If a storage volume entry exists only proceed if force was specified.
+		// If the storage volume entry does already exist we error here
 		if dbVolume != nil {
 			return fmt.Errorf(`Storage volume for snapshot %q already exists in the database`, snapInstName)
 		}
 
-		if snapErr == nil {
-			err := s.DB.Cluster.Transaction(ctx, func(ctx context.Context, tx *db.ClusterTx) error {
-				return tx.DeleteInstance(ctx, projectName, snapInstName)
-			})
-			if err != nil {
-				return err
-			}
-		}
-
-		if dbVolume != nil {
-			err := s.DB.Cluster.Transaction(ctx, func(ctx context.Context, tx *db.ClusterTx) error {
-				return tx.RemoveStoragePoolVolume(ctx, projectName, snapInstName, instanceDBVolType, pool.ID())
-			})
-			if err != nil {
-				return err
-			}
-		}
-
 		baseImage := snap.Config["volatile.base_image"]
 
-		arch, err := osarch.ArchitectureId(snap.Architecture)
+		arch, err := osarch.ArchitectureID(snap.Architecture)
 		if err != nil {
 			return err
 		}
@@ -912,11 +969,11 @@ func internalImportFromBackup(ctx context.Context, s *state.State, projectName s
 
 		// Add root device if needed.
 		if snap.Devices == nil {
-			snap.Devices = make(map[string]map[string]string, 0)
+			snap.Devices = make(map[string]map[string]string)
 		}
 
 		if snap.ExpandedDevices == nil {
-			snap.ExpandedDevices = make(map[string]map[string]string, 0)
+			snap.ExpandedDevices = make(map[string]map[string]string)
 		}
 
 		internalImportRootDevicePopulate(instancePoolName, snap.Devices, snap.ExpandedDevices, profiles)
@@ -926,6 +983,7 @@ func internalImportFromBackup(ctx context.Context, s *state.State, projectName s
 			Architecture: arch,
 			BaseImage:    baseImage,
 			Config:       snap.Config,
+			Description:  snap.Description,
 			CreationDate: snap.CreatedAt,
 			Type:         instanceType,
 			Snapshot:     true,
@@ -940,7 +998,7 @@ func internalImportFromBackup(ctx context.Context, s *state.State, projectName s
 			return fmt.Errorf("Failed creating instance snapshot record %q: %w", snap.Name, err)
 		}
 
-		revert.Add(cleanup)
+		reverter.Add(cleanup)
 		defer snapInstOp.Done(err)
 
 		// Recreate missing mountpoints and symlinks.
@@ -955,7 +1013,7 @@ func internalImportFromBackup(ctx context.Context, s *state.State, projectName s
 		}
 	}
 
-	revert.Success()
+	reverter.Success()
 	return nil
 }
 
@@ -1040,7 +1098,7 @@ func internalImportRootDevicePopulate(instancePoolName string, localDevices map[
 		// If there is already a device called "root" in the instance's config, but it does not qualify as
 		// a root disk, then try to find a free name for the new root disk device.
 		rootDevName := "root"
-		for i := 0; i < 100; i++ {
+		for i := range 100 {
 			if localDevices[rootDevName] == nil {
 				break
 			}
@@ -1053,7 +1111,7 @@ func internalImportRootDevicePopulate(instancePoolName string, localDevices map[
 	}
 }
 
-func internalGC(d *Daemon, r *http.Request) response.Response {
+func internalGC(_ *Daemon, _ *http.Request) response.Response {
 	logger.Infof("Started forced garbage collection run")
 	runtime.GC()
 	runtimeDebug.FreeOSMemory()
@@ -1070,19 +1128,19 @@ func internalGC(d *Daemon, r *http.Request) response.Response {
 	return response.EmptySyncResponse
 }
 
-func internalRAFTSnapshot(d *Daemon, r *http.Request) response.Response {
+func internalRAFTSnapshot(_ *Daemon, _ *http.Request) response.Response {
 	logger.Warn("Forced RAFT snapshot not supported")
 
-	return response.InternalError(fmt.Errorf("Not supported"))
+	return response.InternalError(errors.New("Not supported"))
 }
 
-func internalBGPState(d *Daemon, r *http.Request) response.Response {
+func internalBGPState(d *Daemon, _ *http.Request) response.Response {
 	s := d.State()
 
 	return response.SyncResponse(true, s.BGP.Debug())
 }
 
-func internalRebalanceLoad(d *Daemon, r *http.Request) response.Response {
+func internalRebalanceLoad(d *Daemon, _ *http.Request) response.Response {
 	err := autoRebalanceCluster(context.TODO(), d)
 	if err != nil {
 		return response.SmartError(err)
