@@ -108,6 +108,15 @@ func (d *nicIPVLAN) validateConfig(instConf instance.ConfigReader) error {
 		//  default: false
 		//  shortdesc: Register VLAN using GARP VLAN Registration Protocol
 		"gvrp",
+
+		// gendoc:generate(entity=devices, group=nic_ipvlan, key=attached)
+		//
+		// ---
+		//  type: bool
+		//  default: `true`
+		//  required: no
+		//  shortdesc: Whether the NIC is plugged in or not
+		"attached",
 	}
 
 	rules := nicValidationRules(requiredFields, optionalFields, instConf)
@@ -301,6 +310,11 @@ func (d *nicIPVLAN) validateEnvironment() error {
 
 // Start is run when the instance is starting up (IPVLAN doesn't support hot plugging).
 func (d *nicIPVLAN) Start() (*deviceConfig.RunConfig, error) {
+	// Ignore detached NICs.
+	if !util.IsTrueOrEmpty(d.config["attached"]) {
+		return nil, nil
+	}
+
 	err := d.validateEnvironment()
 	if err != nil {
 		return nil, err
@@ -365,13 +379,13 @@ func (d *nicIPVLAN) Start() (*deviceConfig.RunConfig, error) {
 
 	// Perform network configuration.
 	for _, keyPrefix := range []string{"ipv4", "ipv6"} {
-		var ipFamilyArg string
+		var ipFamily ip.Family
 
 		switch keyPrefix {
 		case "ipv4":
-			ipFamilyArg = ip.FamilyV4
+			ipFamily = ip.FamilyV4
 		case "ipv6":
-			ipFamilyArg = ip.FamilyV6
+			ipFamily = ip.FamilyV6
 		}
 
 		addresses := util.SplitNTrimSpace(d.config[fmt.Sprintf("%s.address", keyPrefix)], ",", -1, true)
@@ -393,9 +407,9 @@ func (d *nicIPVLAN) Start() (*deviceConfig.RunConfig, error) {
 				// Apply host-side static routes to main routing table to allow neighbour proxy.
 				r := ip.Route{
 					DevName: "lo",
-					Route:   addr.String(),
+					Route:   addr,
 					Table:   "main",
-					Family:  ipFamilyArg,
+					Family:  ipFamily,
 				}
 
 				err = r.Add()
@@ -410,9 +424,9 @@ func (d *nicIPVLAN) Start() (*deviceConfig.RunConfig, error) {
 				if d.config[hostTableKey] != "" {
 					r := &ip.Route{
 						DevName: "lo",
-						Route:   addr.String(),
+						Route:   addr,
 						Table:   d.config[hostTableKey],
-						Family:  ipFamilyArg,
+						Family:  ipFamily,
 					}
 
 					err := r.Add()
@@ -540,13 +554,13 @@ func (d *nicIPVLAN) postStop() error {
 
 	// Clean up host-side network configuration.
 	for _, keyPrefix := range []string{"ipv4", "ipv6"} {
-		var ipFamilyArg string
+		var ipFamily ip.Family
 
 		switch keyPrefix {
 		case "ipv4":
-			ipFamilyArg = ip.FamilyV4
+			ipFamily = ip.FamilyV4
 		case "ipv6":
-			ipFamilyArg = ip.FamilyV6
+			ipFamily = ip.FamilyV6
 		}
 
 		addresses := util.SplitNTrimSpace(d.config[fmt.Sprintf("%s.address", keyPrefix)], ",", -1, true)
@@ -563,9 +577,9 @@ func (d *nicIPVLAN) postStop() error {
 			if mode == ipvlanModeL3S {
 				r := ip.Route{
 					DevName: "lo",
-					Route:   addr.String(),
+					Route:   addr,
 					Table:   "main",
-					Family:  ipFamilyArg,
+					Family:  ipFamily,
 				}
 
 				err := r.Delete()
@@ -588,9 +602,9 @@ func (d *nicIPVLAN) postStop() error {
 				if d.config[hostTableKey] != "" {
 					r := &ip.Route{
 						DevName: "lo",
-						Route:   addr.String(),
+						Route:   addr,
 						Table:   d.config[hostTableKey],
-						Family:  ipFamilyArg,
+						Family:  ipFamily,
 					}
 
 					err := r.Delete()
