@@ -35,6 +35,7 @@ import (
 	"github.com/lxc/incus/v6/shared/logger"
 	"github.com/lxc/incus/v6/shared/resources"
 	"github.com/lxc/incus/v6/shared/revert"
+	"github.com/lxc/incus/v6/shared/units"
 	"github.com/lxc/incus/v6/shared/util"
 	"github.com/lxc/incus/v6/shared/validate"
 )
@@ -47,7 +48,7 @@ type ovnNet interface {
 	InstanceDevicePortAdd(instanceUUID string, deviceName string, deviceConfig deviceConfig.Device) error
 	InstanceDevicePortStart(opts *network.OVNInstanceNICSetupOpts, securityACLsRemove []string) (ovn.OVNSwitchPort, []net.IP, error)
 	InstanceDevicePortStop(ovsExternalOVNPort ovn.OVNSwitchPort, opts *network.OVNInstanceNICStopOpts) error
-	InstanceDevicePortRemove(instanceUUID string, deviceName string, deviceConfig deviceConfig.Device) error
+	InstanceDevicePortRemove(instanceUUID string, devName string, devConfig deviceConfig.Device, hasDuplicate bool) error
 	InstanceDevicePortIPs(instanceUUID string, deviceName string) ([]net.IP, error)
 }
 
@@ -78,7 +79,7 @@ func (d *nicOVN) UpdatableFields(oldDevice Type) []string {
 		return []string{}
 	}
 
-	return []string{"security.acls"}
+	return []string{"security.acls", "limits.ingress", "limits.egress", "limits.max", "limits.priority", "connected"}
 }
 
 // validateConfig checks the supplied config for correctness.
@@ -283,6 +284,57 @@ func (d *nicOVN) validateConfig(instConf instance.ConfigReader) error {
 		//  managed: no
 		//  shortdesc: The VLAN ID to use when nesting (see also `nested`)
 		"vlan",
+
+		// gendoc:generate(entity=devices, group=nic_ovn, key=limits.ingress)
+		//
+		// ---
+		//  type: string
+		//  managed: no
+		//  shortdesc: I/O limit in bit/s for incoming traffic (various suffixes supported, see {ref}`instances-limit-units`)
+		"limits.ingress",
+
+		// gendoc:generate(entity=devices, group=nic_ovn, key=limits.egress)
+		//
+		// ---
+		//  type: string
+		//  managed: no
+		//  shortdesc: I/O limit in bit/s for outgoing traffic (various suffixes supported, see {ref}`instances-limit-units`)
+		"limits.egress",
+
+		// gendoc:generate(entity=devices, group=nic_ovn, key=limits.max)
+		//
+		// ---
+		//  type: string
+		//  managed: no
+		//  shortdesc: I/O limit in bit/s for both incoming and outgoing traffic. (same as setting both limits.ingress and limits.egress / mutually exclusive with limits.ingress and limits.egress)
+		"limits.max",
+
+		// gendoc:generate(entity=devices, group=nic_ovn, key=limits.priority)
+		//
+		// ---
+		//  default: 100
+		//  type: integer
+		//  managed: no
+		//  shortdesc: The priority for outgoing traffic, to be used by the kernel queuing discipline to prioritize network packets
+		"limits.priority",
+
+		// gendoc:generate(entity=devices, group=nic_ovn, key=attached)
+		//
+		// ---
+		//  type: bool
+		//  default: `true`
+		//  required: no
+		//  shortdesc: Whether the NIC is plugged in or not
+		"attached",
+
+		// gendoc:generate(entity=devices, group=nic_ovn, key=connected)
+		//
+		// ---
+		//  type: bool
+		//  default: `true`
+		//  required: no
+		//  shortdesc: Whether the NIC is connected to the host network (container support requires setting `acceleration` to `none`)
+		"connected",
 	}
 
 	// The NIC's network may be a non-default project, so lookup project and get network's project name.
@@ -489,6 +541,63 @@ func (d *nicOVN) validateConfig(instConf instance.ConfigReader) error {
 		}
 	}
 
+	// Avoid setting both ingress/egress and max to avoid confusion or implicit behavior.
+	if d.config["limits.max"] != "" && (d.config["limits.ingress"] != "" || d.config["limits.egress"] != "") {
+		return errors.New("limits.max is mutually exclusive with limits.ingress and limits.egress")
+	}
+
+	if d.config["limits.priority"] != "" {
+		priority, err := strconv.Atoi(d.config["limits.priority"])
+		if err != nil {
+			return errors.New("limits.priority must be an integer")
+		}
+
+		if priority < 0 || priority > 32767 {
+			return errors.New("limits.priority must be between 0 an 32767, inclusive")
+		}
+	}
+
+	if d.config["limits.max"] != "" {
+		limitsMax, err := units.ParseBitSizeString(d.config["limits.max"])
+		if err != nil {
+			return errors.New("limits.max must be an integer")
+		}
+
+		limitsMax /= 1000 // Convert to kbps
+		if limitsMax < 1 || limitsMax > 4294967295 {
+			return errors.New("limits.max must be between 1 an 4294967295 bps, inclusive")
+		}
+	}
+
+	if d.config["limits.ingress"] != "" {
+		ingress, err := units.ParseBitSizeString(d.config["limits.ingress"])
+		if err != nil {
+			return errors.New("limits.ingress must be an integer")
+		}
+
+		ingress /= 1000 // Convert to kbps
+		if ingress < 1 || ingress > 4294967295 {
+			return errors.New("limits.ingress must be between 1 an 4294967295 bps, inclusive")
+		}
+	}
+
+	if d.config["limits.egress"] != "" {
+		egress, err := units.ParseBitSizeString(d.config["limits.egress"])
+		if err != nil {
+			return errors.New("limits.egress must be an integer")
+		}
+
+		egress /= 1000 // Convert to kbps
+		if egress < 1 || egress > 4294967295 {
+			return errors.New("limits.egress must be between 1 an 4294967295 bps, inclusive")
+		}
+	}
+
+	// The connected option can only be handled properly on containers if acceleration is set to none.
+	if d.config["connected"] != "" && !d.canSetLink(instConf) {
+		return errors.New("The \"connected\" option requires setting acceleration=none on containers for OVN NICs")
+	}
+
 	return nil
 }
 
@@ -607,6 +716,11 @@ func (d *nicOVN) init(inst instance.Instance, s *state.State, name string, conf
 
 // Start is run when the device is added to a running instance or instance is starting up.
 func (d *nicOVN) Start() (*deviceConfig.RunConfig, error) {
+	// Ignore detached NICs.
+	if !util.IsTrueOrEmpty(d.config["attached"]) {
+		return nil, nil
+	}
+
 	err := d.validateEnvironment()
 	if err != nil {
 		return nil, err
@@ -901,15 +1015,22 @@ func (d *nicOVN) Start() (*deviceConfig.RunConfig, error) {
 			{Key: "link", Value: peerName},
 		}
 
+		if d.canSetLink(nil) {
+			runConf.NetworkInterface = append(runConf.NetworkInterface, deviceConfig.RunConfigItem{Key: "connected", Value: d.config["connected"]})
+		}
+
 		instType := d.inst.Type()
 		if instType == instancetype.VM {
+			runConf.NetworkInterface = append(runConf.NetworkInterface,
+				[]deviceConfig.RunConfigItem{
+					{Key: "devName", Value: d.name},
+					{Key: "mtu", Value: fmt.Sprintf("%d", mtu)},
+				}...)
 			if d.config["acceleration"] == "sriov" {
 				runConf.NetworkInterface = append(runConf.NetworkInterface,
 					[]deviceConfig.RunConfigItem{
-						{Key: "devName", Value: d.name},
 						{Key: "pciSlotName", Value: vfPCIDev.SlotName},
 						{Key: "pciIOMMUGroup", Value: fmt.Sprintf("%d", pciIOMMUGroup)},
-						{Key: "mtu", Value: fmt.Sprintf("%d", mtu)},
 					}...)
 			} else if d.config["acceleration"] == "vdpa" {
 				if vDPADevice == nil {
@@ -918,20 +1039,16 @@ func (d *nicOVN) Start() (*deviceConfig.RunConfig, error) {
 
 				runConf.NetworkInterface = append(runConf.NetworkInterface,
 					[]deviceConfig.RunConfigItem{
-						{Key: "devName", Value: d.name},
 						{Key: "pciSlotName", Value: vfPCIDev.SlotName},
 						{Key: "pciIOMMUGroup", Value: fmt.Sprintf("%d", pciIOMMUGroup)},
 						{Key: "maxVQP", Value: fmt.Sprintf("%d", vDPADevice.MaxVQs/2)},
 						{Key: "vDPADevName", Value: vDPADevice.Name},
 						{Key: "vhostVDPAPath", Value: vDPADevice.VhostVDPA.Path},
-						{Key: "mtu", Value: fmt.Sprintf("%d", mtu)},
 					}...)
 			} else {
 				runConf.NetworkInterface = append(runConf.NetworkInterface,
 					[]deviceConfig.RunConfigItem{
-						{Key: "devName", Value: d.name},
 						{Key: "hwaddr", Value: d.config["hwaddr"]},
-						{Key: "mtu", Value: fmt.Sprintf("%d", mtu)},
 					}...)
 			}
 		} else if instType == instancetype.Container {
@@ -1056,6 +1173,10 @@ func (d *nicOVN) Update(oldDevices deviceConfig.Devices, isRunning bool) error {
 		return err
 	}
 
+	if isRunning && d.canSetLink(nil) {
+		return d.setNICLink()
+	}
+
 	return nil
 }
 
@@ -1247,7 +1368,7 @@ func (d *nicOVN) Remove() error {
 		}
 	}
 
-	return d.network.InstanceDevicePortRemove(d.inst.LocalConfig()["volatile.uuid"], d.name, d.config)
+	return d.network.InstanceDevicePortRemove(d.inst.LocalConfig()["volatile.uuid"], d.name, d.config, d.checkAddressConflict() != nil)
 }
 
 // State gets the state of an OVN NIC by querying the OVN Northbound logical switch port record.
@@ -1448,3 +1569,15 @@ func (d *nicOVN) setupHostNIC(hostName string, ovnPortName ovn.OVNSwitchPort) (r
 
 	return cleanup, err
 }
+
+// canSetLink determines whether the device supports setting a link state.
+func (d *nicOVN) canSetLink(instConf instance.ConfigReader) bool {
+	var instType instancetype.Type
+	if instConf != nil {
+		instType = instConf.Type()
+	} else {
+		instType = d.inst.Type()
+	}
+
+	return instType == instancetype.VM || slices.Contains([]string{"", "none"}, d.config["acceleration"])
+}
