@@ -867,7 +867,7 @@ func (m *Monitor) RemoveDevice(deviceID string) error {
 }
 
 // AddNIC adds a NIC device.
-func (m *Monitor) AddNIC(netDev map[string]any, device map[string]any) error {
+func (m *Monitor) AddNIC(netDev map[string]any, device map[string]any, connected bool) error {
 	reverter := revert.New()
 	defer reverter.Fail()
 
@@ -894,6 +894,16 @@ func (m *Monitor) AddNIC(netDev map[string]any, device map[string]any) error {
 		return fmt.Errorf("Failed adding NIC device: %w", err)
 	}
 
+	id, ok := device["id"].(string)
+	if !ok {
+		return errors.New("NIC device must have an id")
+	}
+
+	err = m.SetNICLink(id, connected)
+	if err != nil {
+		return fmt.Errorf("Failed setting NIC device link status: %w", err)
+	}
+
 	reverter.Success()
 
 	return nil
@@ -1128,6 +1138,47 @@ func (m *Monitor) NBDBlockExportAdd(deviceNodeName string) error {
 	return nil
 }
 
+// QueryNamedBlockNodes returns block nodes names.
+func (m *Monitor) QueryNamedBlockNodes() ([]string, error) {
+	var resp struct {
+		Return []struct {
+			NodeName string `json:"node-name"`
+		} `json:"return"`
+	}
+
+	err := m.Run("query-named-block-nodes", nil, &resp)
+	if err != nil {
+		return nil, err
+	}
+
+	result := []string{}
+	for _, r := range resp.Return {
+		result = append(result, r.NodeName)
+	}
+
+	return result, nil
+}
+
+// ChangeBackingFile changes backing file name for node.
+func (m *Monitor) ChangeBackingFile(deviceNodeName string, imageNodeName string, backingFilename string) error {
+	var args struct {
+		Device      string `json:"device"`
+		NodeName    string `json:"image-node-name"`
+		BackingFile string `json:"backing-file"`
+	}
+
+	args.Device = deviceNodeName
+	args.NodeName = imageNodeName
+	args.BackingFile = backingFilename
+
+	err := m.Run("change-backing-file", args, nil)
+	if err != nil {
+		return err
+	}
+
+	return nil
+}
+
 // BlockDevSnapshot creates a snapshot of a device using the specified snapshot device.
 func (m *Monitor) BlockDevSnapshot(deviceNodeName string, snapshotNodeName string) error {
 	var args struct {
@@ -1148,7 +1199,7 @@ func (m *Monitor) BlockDevSnapshot(deviceNodeName string, snapshotNodeName strin
 
 // blockJobWaitReady waits until the specified jobID is ready, errored or missing.
 // Returns nil if the job is ready, otherwise an error.
-func (m *Monitor) blockJobWaitReady(jobID string) error {
+func (m *Monitor) blockJobWaitReady(jobID string, exitOnNotFound bool) error {
 	for {
 		var resp struct {
 			Return []struct {
@@ -1180,8 +1231,10 @@ func (m *Monitor) blockJobWaitReady(jobID string) error {
 			found = true
 		}
 
-		if !found {
+		if !found && !exitOnNotFound {
 			return errors.New("Specified block job not found")
+		} else if !found && exitOnNotFound {
+			return nil
 		}
 
 		time.Sleep(1 * time.Second)
@@ -1189,28 +1242,37 @@ func (m *Monitor) blockJobWaitReady(jobID string) error {
 }
 
 // BlockCommit merges a snapshot device back into its parent device.
-func (m *Monitor) BlockCommit(deviceNodeName string) error {
+func (m *Monitor) BlockCommit(deviceNodeName string, top string, base string) error {
 	var args struct {
 		Device string `json:"device"`
 		JobID  string `json:"job-id"`
+		Top    string `json:"top-node"`
+		Base   string `json:"base-node"`
 	}
 
 	args.Device = deviceNodeName
 	args.JobID = args.Device
+	args.Top = top
+	args.Base = base
 
 	err := m.Run("block-commit", args, nil)
 	if err != nil {
 		return err
 	}
 
-	err = m.blockJobWaitReady(args.JobID)
+	// From QEMU doc: If top has no overlays on top of it, or if it is in use by a writer,
+	// the job will not be completed by itself.
+	hasTop := top != deviceNodeName && top != ""
+	err = m.blockJobWaitReady(args.JobID, hasTop)
 	if err != nil {
 		return err
 	}
 
-	err = m.BlockJobComplete(args.JobID)
-	if err != nil {
-		return err
+	if !hasTop {
+		err = m.BlockJobComplete(args.JobID)
+		if err != nil {
+			return err
+		}
 	}
 
 	return nil
@@ -1243,7 +1305,7 @@ func (m *Monitor) BlockDevMirror(deviceNodeName string, targetNodeName string) e
 		return err
 	}
 
-	err = m.blockJobWaitReady(args.JobID)
+	err = m.blockJobWaitReady(args.JobID, false)
 	if err != nil {
 		return err
 	}
@@ -1284,14 +1346,14 @@ func (m *Monitor) BlockJobComplete(deviceNodeName string) error {
 }
 
 // UpdateBlockSize updates the size of a disk.
-func (m *Monitor) UpdateBlockSize(id string) error {
+func (m *Monitor) UpdateBlockSize(id string, size int64) error {
 	var args struct {
 		NodeName string `json:"node-name"`
 		Size     int64  `json:"size"`
 	}
 
 	args.NodeName = id
-	args.Size = 1
+	args.Size = size
 
 	err := m.Run("block_resize", args, nil)
 	if err != nil {
@@ -1510,3 +1572,15 @@ func (m *Monitor) DumpGuestMemory(path string, format string) error {
 
 	return m.Run("dump-guest-memory", args, &queryResp)
 }
+
+// SetNICLink sets the link status of the given device.
+func (m *Monitor) SetNICLink(id string, connected bool) error {
+	var args struct {
+		Name string `json:"name"`
+		Up   bool   `json:"up"`
+	}
+
+	args.Name = id
+	args.Up = connected
+	return m.Run("set_link", args, nil)
+}
