@@ -57,6 +57,28 @@ func (n *physical) Validate(config map[string]string, clientType request.ClientT
 		// shortdesc: The VLAN ID to attach to
 		"vlan": validate.Optional(validate.IsNetworkVLAN),
 
+		// gendoc:generate(entity=network_physical, group=common, key=vlan.tagged)
+		//
+		// ---
+		// type: integer
+		// condition: Parent must be an existing bridge
+		// shortdesc: Comma-delimited list of VLAN IDs or VLAN ranges to join for tagged traffic
+		"vlan.tagged": func(value string) error {
+			if value == "" {
+				return nil
+			}
+
+			// Check that none of the supplied VLAN IDs are the same as the untagged VLAN ID.
+			for _, vlanID := range util.SplitNTrimSpace(value, ",", -1, true) {
+				_, _, err := validate.ParseNetworkVLANRange(vlanID)
+				if err != nil {
+					return err
+				}
+			}
+
+			return nil
+		},
+
 		// gendoc:generate(entity=network_physical, group=common, key=gvrp)
 		//
 		// ---
@@ -217,6 +239,11 @@ func (n *physical) Validate(config map[string]string, clientType request.ClientT
 
 // checkParentUse checks if parent is already in use by another network or instance device.
 func (n *physical) checkParentUse(ourConfig map[string]string) (bool, error) {
+	// Check if we're dealing with a physical network backed by a bridge.
+	if ourConfig["parent"] != "" && util.PathExists(fmt.Sprintf("/sys/class/net/%s/bridge", ourConfig["parent"])) {
+		return false, nil
+	}
+
 	// Get all managed networks across all projects.
 	var err error
 	var projectNetworks map[string]map[int64]api.Network
@@ -502,12 +529,15 @@ func (n *physical) Update(newNetwork api.NetworkPut, targetNode string, clientTy
 		if err == nil {
 			ovsBridge := fmt.Sprintf("incusovn%d", n.id)
 
-			err := vswitch.DeleteBridgePort(context.TODO(), ovsBridge, oldNetwork.Config["parent"])
+			oldHostName := GetHostDevice(oldNetwork.Config["parent"], oldNetwork.Config["vlan"])
+			newHostName := GetHostDevice(newNetwork.Config["parent"], newNetwork.Config["vlan"])
+
+			err := vswitch.DeleteBridgePort(context.TODO(), ovsBridge, oldHostName)
 			if err != nil && !errors.Is(err, ovs.ErrNotFound) {
 				return err
 			}
 
-			err = vswitch.CreateBridgePort(context.TODO(), ovsBridge, newNetwork.Config["parent"], true)
+			err = vswitch.CreateBridgePort(context.TODO(), ovsBridge, newHostName, true)
 			if err != nil && !errors.Is(err, ovs.ErrNotFound) {
 				return err
 			}
