@@ -511,6 +511,12 @@ var APIExtensions = []string{
 	"device_pci_firmware",
 	"resources_serial",
 	"ovn_nic_limits",
+	"storage_lvmcluster_qcow2",
+	"oidc_allowed_subnets",
+	"file_delete_force",
+	"nic_sriov_select_ext",
+	"network_zones_dns_contact",
+	"nic_attached_connected",
 }
 
 // APIExtensionsCount returns the number of available API extensions.
