@@ -2,6 +2,7 @@ package validate
 
 import (
 	"bytes"
+	"encoding/base64"
 	"errors"
 	"fmt"
 	"net"
@@ -13,6 +14,7 @@ import (
 	"strconv"
 	"strings"
 	"time"
+	"unicode"
 
 	"github.com/adhocore/gronx"
 	"github.com/google/uuid"
@@ -148,6 +150,20 @@ func IsPriority(value string) error {
 	return nil
 }
 
+// IsOOMPriority validates priority number.
+func IsOOMPriority(value string) error {
+	valueInt, err := strconv.ParseInt(value, 10, 64)
+	if err != nil {
+		return fmt.Errorf("Invalid value for an integer %q", value)
+	}
+
+	if valueInt < -1000 || valueInt > 1000 {
+		return fmt.Errorf("Invalid value for a limit %q. Must be between -1000 and 1000", value)
+	}
+
+	return nil
+}
+
 // IsBool validates if string can be understood as a bool.
 func IsBool(value string) error {
 	if !slices.Contains([]string{"true", "false", "yes", "no", "1", "0", "on", "off"}, strings.ToLower(value)) {
@@ -238,25 +254,6 @@ func IsInterfaceName(value string) error {
 	return nil
 }
 
-// IsNetworkName validates a name usable for a network.
-func IsNetworkName(value string) error {
-	err := IsInterfaceName(value)
-	if err != nil {
-		return err
-	}
-
-	err = IsURLSegmentSafe(value)
-	if err != nil {
-		return err
-	}
-
-	if strings.Contains(value, ":") {
-		return fmt.Errorf("Cannot contain %q", ":")
-	}
-
-	return nil
-}
-
 // IsNetworkMAC validates an Ethernet MAC address. e.g. "00:00:5e:00:53:01".
 func IsNetworkMAC(value string) error {
 	_, err := net.ParseMAC(value)
@@ -269,6 +266,16 @@ func IsNetworkMAC(value string) error {
 	return nil
 }
 
+// IsMACPattern validates whether the string is a MAC address pattern.
+func IsMACPattern(value string) error {
+	match, _ := regexp.MatchString(`^(?:[0-9a-fA-FxX]{2}:){5}[0-9a-fA-FxX]{2}$`, value)
+	if !match {
+		return errors.New("Invalid MAC address pattern, must be 6 bytes (hex characters or `x` to denote a wildcard) separated by colons")
+	}
+
+	return nil
+}
+
 // IsNetworkAddress validates an IP (v4 or v6) address string.
 func IsNetworkAddress(value string) error {
 	ip := net.ParseIP(value)
@@ -552,14 +559,42 @@ func IsDHCPRouteList(value string) error {
 	return nil
 }
 
-// IsURLSegmentSafe validates whether value can be used in a URL segment.
-func IsURLSegmentSafe(value string) error {
-	for _, char := range []string{"/", "?", "&", "+"} {
-		if strings.Contains(value, char) {
-			return fmt.Errorf("Cannot contain %q", char)
+// IsAPIName checks whether the provided value is a suitable name for an API object.
+func IsAPIName(value string, allowSlashes bool) error {
+	// Limit length to 64 characters.
+	if len(value) > 64 {
+		return errors.New("Maximum name length is 64 characters")
+	}
+
+	// Check for unicode characters.
+	for _, r := range value {
+		if unicode.IsSpace(r) {
+			return errors.New("Name cannot contain white space")
 		}
 	}
 
+	// Check for special URL characters.
+	reservedChars := []string{"$", "?", "&", "+", "\"", "'", "`", "*"}
+	if !allowSlashes {
+		reservedChars = append(reservedChars, "/")
+	}
+
+	for _, char := range reservedChars {
+		if strings.Contains(value, char) {
+			return fmt.Errorf("Name contains invalid character %q", char)
+		}
+	}
+
+	// Check beginning and end.
+	match, err := regexp.MatchString(`^[a-zA-Z0-9]+.*[a-zA-Z0-9]+$`, value)
+	if err != nil {
+		return err
+	}
+
+	if !match {
+		return errors.New("Names must start and end with an alphanumeric character")
+	}
+
 	return nil
 }
 
@@ -918,3 +953,13 @@ func IsMinimumDuration(minimum time.Duration) func(value string) error {
 		return nil
 	}
 }
+
+// IsBase64 validates whether the string is Base64 encoded.
+func IsBase64(value string) error {
+	_, err := base64.RawStdEncoding.DecodeString(strings.TrimRight(value, "="))
+	if err != nil {
+		return fmt.Errorf("Invalid value for a base64 string %q: %w", value, err)
+	}
+
+	return nil
+}
